@@ -276,7 +276,7 @@ kernel_dispersal <- function (dispersal_kernel = exponential_dispersal_kernel(di
     # store the original population, so that individuals don't disperse twice
     original_pop <- pop
     
-    # loop through origins and stages where there is at least one individual    
+    # index all pop origins and stages where there is at least one individual    
     indices <- which(pop > 0 & !is.na(pop), arr.ind = TRUE)
     
     # subset to stages that disperse
@@ -316,12 +316,14 @@ kernel_dispersal <- function (dispersal_kernel = exponential_dispersal_kernel(di
 #' The cellular_automata_dispersal function simulates movements of
 #' individuals using rule-based cell movements. In each cell that has
 #' population, every individual up to a specified proportion of the
-#' total population attempts to move. For each step up to a specified
-#' maximum number of movements, a weighted draw of four directions, based on
-#' habitat suitability, is made and then the destination cell is checked
-#' for available carrying capacity. If there is carrying capacity available,
-#' the individual moves to the cell, if not, it remains in its current cell.
-#' This is repeated until the maximum number of cell movements is reached. 
+#' total population attempts to move. For each step from a specified minimum up
+#' to a specified maximum number of movements, a weighted draw of four
+#' directions, based on habitat suitability, is made and then the destination
+#' cell is checked for available carrying capacity. If there is carrying capacity
+#' available, the individual moves to the cell, if not, it remains in its current
+#' cell. This is repeated until the maximum number of cell movements is reached.
+#' If no cell is found with available carrying capacity, the individual remains
+#' in the source cell.
 #' 
 #' This function allows the use of barriers in the landscape to influence
 #' dispersal. The function is computationally efficient, however, because
@@ -331,20 +333,23 @@ kernel_dispersal <- function (dispersal_kernel = exponential_dispersal_kernel(di
 #' The maximum number of cell movements in cellular automata dispersal does not
 #' correspond exactly to the distance decay of a dispersal kernel, since cellular
 #' automata dispersal depends on the permeability of the landscape, and is
-#' interrupted on reaching a cell with available capacity. A heuristic that can be
-#' used to determine a reasonable number of steps from a mean dispersal distance `d`
-#' and cell resolution `res` is: `max_cells = round(2 * (d / (res * 1.25)) ^ 2)`.
-#' This corresponds approximately to the number of cell-steps in an infinite,
+#' interrupted on reaching a cell with available capacity (above the minimum
+#' specified number of cell movements). A heuristic that can be used to determine
+#' a reasonable number of steps from a mean dispersal distance `d` and cell
+#' resolution `res` is: `max_cells = round(2 * (d / (res * 1.25)) ^ 2)`. This
+#' corresponds approximately to the number of cell-steps in an infinite,
 #' homogenous landscape with no early stopping, for which d is the mean
 #' end-to-end dispersal distance of all individuals.
 #'
 #' Rather than relying on this value, we recommend that the user experiment with
-#' the \code{max_cells} parameter to find a value such that the the mean dispersal
-#' distance in a reasonably realistic simulation corresponds with field estimates
-#' of mean dispersal distances.
+#' the \code{max_cells} and \code{min_cells} parameters to find a value such that
+#' the the mean dispersal distance in a reasonably realistic simulation
+#' corresponds with field estimates of mean dispersal distances.
 #'
 #' @param max_cells the maximum number of cell movements that each individual in
 #'   each life stage can disperse in whole integers.
+#' @param min_cells the minimum number of cell movements that each individual in
+#'   each life stage will disperse in whole integers.
 #' @param dispersal_proportion a built-in or custom function defining the proportions
 #'   of individuals that can disperse in each life stage.
 #' @param barriers the name of a spatial layer in the landscape object that
@@ -383,6 +388,7 @@ kernel_dispersal <- function (dispersal_kernel = exponential_dispersal_kernel(di
 #' }
 
 cellular_automata_dispersal <- function (max_cells = Inf,
+                                         min_cells = max_cells, ### Added 07.02.22
                                          dispersal_proportion = set_proportion_dispersing(),
                                          barriers = NULL,
                                          use_suitability = TRUE,
@@ -425,6 +431,11 @@ cellular_automata_dispersal <- function (max_cells = Inf,
       cc <- landscape[[carrying_capacity]]
     }
     
+    # do the minimum and maximum values have the same length? - Added 07.02.22
+    if (length(min_cells) != length(max_cells)) {
+      stop("Minimum cells and maximum cells must have the same length (i.e. number of life stages)")
+    }
+    
     # get population as a matrix
     population <- raster::extract(population_raster, idx)
     
@@ -441,7 +452,8 @@ cellular_automata_dispersal <- function (max_cells = Inf,
       n_rows <- raster::nrow(population_raster[[1]])
       n_cols <- raster::ncol(population_raster[[1]])
       res <- raster::res(population_raster[[1]])
-      max_cells <- sqrt( (n_cols * res[1])^2 + (n_rows * res[2])^2 )
+      max_cells <- round(2 * (max(n_rows, n_cols) / (res * 1.25)) ^ 2)
+      min_cells <- max_cells ### Added 07.02.22
     }
     
     # handle dispersal distances as both scalars and vectors
@@ -454,14 +466,25 @@ cellular_automata_dispersal <- function (max_cells = Inf,
                       paste(max_cells, collapse = ", "),
                       "were specified.\nAll life stages will use this distance."),
                 warning_name = "dispersal_distances")
+      warn_once(length(min_cells) < n_stages | length(min_cells) > n_stages, ### Added 07.02.22
+                paste(n_stages,
+                      "life stages exist but",
+                      length(min_cells),
+                      "minimum cell movement(s) of",
+                      paste(min_cells, collapse = ", "),
+                      "were specified.\nAll life stages will use this distance."),
+                warning_name = "dispersal_distances")
     }
     
     if (length(max_cells) < n_stages | length(max_cells) > n_stages) {
       max_cells <- rep_len(max_cells, n_stages)
     }
+    if (length(min_cells) < n_stages | length(min_cells) > n_stages) { ### Added 07.02.22
+      min_cells <- rep_len(min_cells, n_stages)
+    }
     
     # identify dispersing stages
-    which_stages_disperse <- which(dispersal_proportion > 0 & max_cells > 0)
+    which_stages_disperse <- which(dispersal_proportion > 0 & max_cells > 0 & min_cells > 0) ### Revised 07.02.22
     
     # if no barrier map is specified, create a barriers matrix with all zeros.
     if (is.null(barriers)) {
@@ -491,6 +514,7 @@ cellular_automata_dispersal <- function (max_cells = Inf,
                                   raster::as.matrix(cc),
                                   raster::as.matrix(permeability_map),
                                   as.integer(max_cells[i]),
+                                  as.integer(min_cells[i]), ### Added 07.02.22
                                   as.numeric(dispersal_proportion[i]))
       
       population_raster[[i]][] <- dispersed$future_population
@@ -777,7 +801,7 @@ dispersalFFT <- function (popmat, fs) {
 
 seq_range <- function (range, by = 1) seq(range[1], range[2], by = by)
 
-# compute the *relative* distances to all neightbouring cells within a maximum
+# compute the *relative* distances to all neighbouring cells within a maximum
 # distance
 get_distance_info <- function(res, max_distance) {
   
@@ -791,6 +815,7 @@ get_distance_info <- function(res, max_distance) {
   keep <- dists < max_distance
   
   # relative coordinates of cells that are within the distance
+  # ur <- cell_coord[keep, , drop = FALSE]
   ur <- cell_coord[keep, ]
   ul <- cbind(-ur[, 1], ur[, 2])
   ll <- -ur
@@ -806,16 +831,12 @@ get_distance_info <- function(res, max_distance) {
 
 # given a cell id, find the coordinates (in number of cells from the origin)
 id2coord <- function (id, dim) {
-  # get coordinates in cell numbers. 
-  # index from 0
-  id0 <- id - 1
-  
   # how many rows have been covered
-  row0 <- id0 %/% dim[1]
+  row <- id %/% dim[2]
   # how many columns have been covered in the most recent row
-  col0 <- id0 %% dim[1]
-  # combine and switch back to indexing from 1
-  coord <- cbind(col0, row0) + 1
+  col <- id %% dim[2]
+  # combine
+  coord <- cbind(col, (row + 1))
   # make sure they are within the raster (for completeness)
   max_id <- prod(dim)
   invalid_id <- id > max_id | id < 1
@@ -907,7 +928,8 @@ disperse <- function (origin,
   # probability of dispersing multiplied by probability of arrival
   prob <- prob * arrival_prob_values[destination_ids]
   
-  # standardise probabilities
+  # standardise probabilities & check for NaN values
+  # prob <- ifelse(!prob, 0, prob / sum(prob))
   prob <- prob / sum(prob)
   
   # get number dispersing and staying
